var path = require('path');
var os = require('os');
var fs = require('graceful-fs');
var xdgBasedir = require('xdg-basedir');
var writeFileAtomic = require('write-file-atomic');
var dotProp = require('dot-prop');
var uniqueString = require('unique-string');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var path__default = /*#__PURE__*/_interopDefault(path);
var os__default = /*#__PURE__*/_interopDefault(os);
var fs__default = /*#__PURE__*/_interopDefault(fs);
var writeFileAtomic__default = /*#__PURE__*/_interopDefault(writeFileAtomic);
var uniqueString__default = /*#__PURE__*/_interopDefault(uniqueString);

const configDirectory = xdgBasedir.xdgConfig || path__default.default.join(os__default.default.tmpdir(), uniqueString__default.default());
const permissionError = 'You don\'t have access to this file.';
const mkdirOptions = {mode: 0o0700, recursive: true};
const writeFileOptions = {mode: 0o0600};

class Configstore {
	constructor(id, defaults, options = {}) {
		const pathPrefix = options.globalConfigPath ?
			path__default.default.join(id, 'config.json') :
			path__default.default.join('configstore', `${id}.json`);

		this._path = options.configPath || path__default.default.join(configDirectory, pathPrefix);

		if (defaults) {
			this.all = {
				...defaults,
				...this.all
			};
		}
	}

	get all() {
		try {
			return JSON.parse(fs__default.default.readFileSync(this._path, 'utf8'));
		} catch (error) {
			// Create directory if it doesn't exist
			if (error.code === 'ENOENT') {
				return {};
			}

			// Improve the message of permission errors
			if (error.code === 'EACCES') {
				error.message = `${error.message}\n${permissionError}\n`;
			}

			// Empty the file if it encounters invalid JSON
			if (error.name === 'SyntaxError') {
				writeFileAtomic__default.default.sync(this._path, '', writeFileOptions);
				return {};
			}

			throw error;
		}
	}

	set all(value) {
		try {
			// Make sure the folder exists as it could have been deleted in the meantime
			fs__default.default.mkdirSync(path__default.default.dirname(this._path), mkdirOptions);

			writeFileAtomic__default.default.sync(this._path, JSON.stringify(value, undefined, '\t'), writeFileOptions);
		} catch (error) {
			// Improve the message of permission errors
			if (error.code === 'EACCES') {
				error.message = `${error.message}\n${permissionError}\n`;
			}

			throw error;
		}
	}

	get size() {
		return Object.keys(this.all || {}).length;
	}

	get(key) {
		return dotProp.getProperty(this.all, key);
	}

	set(key, value) {
		const config = this.all;

		if (arguments.length === 1) {
			for (const k of Object.keys(key)) {
				dotProp.setProperty(config, k, key[k]);
			}
		} else {
			dotProp.setProperty(config, key, value);
		}

		this.all = config;
	}

	has(key) {
		return dotProp.hasProperty(this.all, key);
	}

	delete(key) {
		const config = this.all;
		dotProp.deleteProperty(config, key);
		this.all = config;
	}

	clear() {
		this.all = {};
	}

	get path() {
		return this._path;
	}
}

module.exports = Configstore;
